@@ -66,7 +66,6 @@ import Data.Aeson
 import Data.Aeson.Types (Pair, parseMaybe, parseEither, Parser)
 import Text.Libyaml hiding (encode, decode, encodeFile, decodeFile)
 import Data.ByteString (ByteString)
-import qualified Data.ByteString.Char8 as S8
 import qualified Data.Map as Map
 import System.IO.Unsafe (unsafePerformIO)
 import Control.Exception (try, throwIO, fromException, Exception, SomeException, AsyncException)
@@ -80,21 +79,31 @@ import Data.Char (toUpper)
 import qualified Data.Vector as V
 import Data.Text (Text, pack)
 import qualified Data.Text as T
-import Data.Text.Read (signed, decimal, double)
 import Data.Text.Encoding (encodeUtf8, decodeUtf8With)
 import Data.Text.Encoding.Error (lenientDecode)
 import qualified Data.HashMap.Strict as M
 import Data.Typeable
-import Data.Attoparsec.Number
 import qualified Data.HashSet as HashSet
+import Data.Text.Read
+#if MIN_VERSION_aeson(0, 7, 0)
+import Data.Scientific (fromFloatDigits)
+import qualified Data.Text.Encoding as TE
+import qualified Data.Text.Lazy as TL
+import Data.Text.Lazy.Builder (toLazyText)
+import Data.Aeson.Encode (encodeToTextBuilder)
+#else
+import Data.Attoparsec.Number
+import qualified Data.ByteString.Char8 as S8
+#endif
+import Control.Monad.Trans.Resource (ResourceT, runResourceT)
 
 encode :: ToJSON a => a -> ByteString
 encode obj = unsafePerformIO $
-    C.runResourceT $ CL.sourceList (objToEvents $ toJSON obj)
+    runResourceT $ CL.sourceList (objToEvents $ toJSON obj)
                 C.$$ Y.encode
 
 encodeFile :: ToJSON a => FilePath -> a -> IO ()
-encodeFile fp obj = C.runResourceT
+encodeFile fp obj = runResourceT
             $ CL.sourceList (objToEvents $ toJSON obj)
          C.$$ Y.encodeFile fp
 
@@ -130,12 +139,17 @@ objToEvents' (String s) rest =
     event
         -- Make sure that special strings are encoded as strings properly.
         -- See: https://github.com/snoyberg/yaml/issues/31
-        | s `HashSet.member` specialStrings = EventScalar (encodeUtf8 s) NoTag SingleQuoted Nothing
+        | s `HashSet.member` specialStrings || isNumeric s = EventScalar (encodeUtf8 s) NoTag SingleQuoted Nothing
         | otherwise = EventScalar (encodeUtf8 s) StrTag PlainNoTag Nothing
 objToEvents' Null rest = EventScalar "null" NullTag PlainNoTag Nothing : rest
 objToEvents' (Bool True) rest = EventScalar "true" BoolTag PlainNoTag Nothing : rest
 objToEvents' (Bool False) rest = EventScalar "false" BoolTag PlainNoTag Nothing : rest
+#if MIN_VERSION_aeson(0,7,0)
+-- Use aeson's implementation which gets rid of annoying decimal points
+objToEvents' n@Number{} rest = EventScalar (TE.encodeUtf8 $ TL.toStrict $ toLazyText $ encodeToTextBuilder n) IntTag PlainNoTag Nothing : rest
+#else
 objToEvents' (Number n) rest = EventScalar (S8.pack $ show n) IntTag PlainNoTag Nothing : rest
+#endif
 
 pairToEvents :: Pair -> [Y.Event] -> [Y.Event]
 pairToEvents (k, v) rest =
@@ -147,6 +161,16 @@ specialStrings :: HashSet.HashSet Text
 specialStrings = HashSet.fromList $ T.words
     "y Y yes Yes YES n N no No NO true True TRUE false False FALSE on On ON off Off OFF null Null NULL ~"
 
+isNumeric :: Text -> Bool
+isNumeric =
+    T.all isNumeric'
+  where
+    isNumeric' c = ('0' <= c && c <= '9')
+                || c == 'e'
+                || c == 'E'
+                || c == '.'
+                || c == '-'
+
 -- Parsing
 
 data ParseException = NonScalarKey
@@ -173,7 +197,7 @@ instance MonadTrans PErrorT where
 instance MonadIO m => MonadIO (PErrorT m) where
     liftIO = lift . liftIO
 
-type Parse = StateT (Map.Map String Value) (C.ResourceT IO)
+type Parse = StateT (Map.Map String Value) (ResourceT IO)
 
 requireEvent :: Event -> C.Sink Event Parse ()
 requireEvent e = do
@@ -210,8 +234,13 @@ textToValue _ _ t
     | t `elem` ["null", "Null", "NULL", "~", ""] = Null
     | any (t `isLike`) ["y", "yes", "on", "true"] = Bool True
     | any (t `isLike`) ["n", "no", "off", "false"] = Bool False
+#if MIN_VERSION_aeson(0, 7, 0)
+    | Right (x, "") <- signed decimal t = Number $ fromIntegral (x :: Integer)
+    | Right (x, "") <- double t = Number $ fromFloatDigits x
+#else
     | Right (x, "") <- signed decimal t = Number $ I x
     | Right (x, "") <- double t = Number $ D x
+#endif
     | otherwise = String t
   where x `isLike` ref = x `elem` [ref, T.toUpper ref, titleCased]
           where titleCased = toUpper (T.head ref) `T.cons` T.tail ref
@@ -301,7 +330,7 @@ decodeFileEither
     => FilePath
     -> IO (Either ParseException a)
 decodeFileEither fp = do
-    x <- try $ C.runResourceT $ flip evalStateT Map.empty $ Y.decodeFile fp C.$$ parse
+    x <- try $ runResourceT $ flip evalStateT Map.empty $ Y.decodeFile fp C.$$ parse
     case x of
         Left e
             | Just pe <- fromException e -> return $ Left pe
@@ -337,7 +366,7 @@ decodeHelper :: FromJSON a
              => C.Source Parse Y.Event
              -> IO (Either ParseException (Either String a))
 decodeHelper src = do
-    x <- try $ C.runResourceT $ flip evalStateT Map.empty $ src C.$$ parse
+    x <- try $ runResourceT $ flip evalStateT Map.empty $ src C.$$ parse
     case x of
         Left e
             | Just pe <- fromException e -> return $ Left pe
